@@ -23,6 +23,8 @@
 #include <config.h>
 #endif
 
+#include <arpa/inet.h>
+
 #define VERSION_NUMBER PACKAGE_VERSION
 
 #include <stdio.h>
@@ -157,20 +159,37 @@ struct smartarray_id_t {
 	{ 0x3354103C, "Smart Array P420i", 0, 1},
 	{ 0x3355103C, "Smart Array P220i", 0, 1},
 	{ 0x3356103C, "Smart Array P721m", 0, 1},
-	{ 0x1920103C, "Smart Array", 0, 1},
-	{ 0x1921103C, "Smart Array", 0, 1},
-	{ 0x1922103C, "Smart Array", 0, 1},
-	{ 0x1923103C, "Smart Array", 0, 1},
-	{ 0x1924103C, "Smart Array", 0, 1},
-	{ 0x1925103C, "Smart Array", 0, 1},
-	{ 0x1926103C, "Smart Array", 0, 1},
-	{ 0x1928103C, "Smart Array", 0, 1},
+	{ 0x1920103C, "Smart Array P430i", 0, 1},
+	{ 0x1921103C, "Smart Array P830i", 0, 1},
+	{ 0x1922103C, "Smart Array P430", 0, 1},
+	{ 0x1923103C, "Smart Array P431", 0, 1},
+	{ 0x1924103C, "Smart Array P830", 0, 1},
+	{ 0x1925103C, "Smart Array P831", 0, 1},
+	{ 0x1926103C, "Smart Array P731m", 0, 1},
+	{ 0x1928103C, "Smart Array P230i", 0, 1},
+	{ 0x1929103C, "Smart Array P530", 0, 1},
+	{ 0x192A103C, "Smart Array P531", 0, 1},
 	{ 0x334D103C, "Smart Array P822se", 0, 1},
 	{ 0x00451590, "Dynamic Smart Array B320i", 0, 1},
 	{ 0x00471590, "Dynamic Smart Array B320i", 0, 1}, 
 	{ 0x00481590, "Dynamic Smart Array B120i", 0, 1},
 	{ 0x006C1590, "Dynamic Smart Array B120i", 0, 1},
 	{ 0x00841590, "Dynamic Smart Array B120i", 0, 1},
+	{ 0x21BD103C, "Smart Array P244br", 0, 1},
+	{ 0x21BE103C, "Smart Array P741m", 0, 1},
+	{ 0x21BF103C, "Smart HBA H240ar", 0, 1},
+	{ 0x21C0103C, "Smart Array P440ar", 0, 1},
+	{ 0x21C2103C, "Smart Array P440", 0, 1},
+	{ 0x21C3103C, "Smart Array P441", 0, 1},
+	{ 0x21C5103C, "Smart Array P841", 0, 1},
+	{ 0x21C6103C, "Smart HBA H244br", 0, 1},
+	{ 0x21C7103C, "Smart HBA H240", 0, 1},
+	{ 0x21C8103C, "Smart HBA H241", 0, 1},
+	{ 0x21CA103C, "Smart Array P246br", 0, 1},
+	{ 0x21CB103C, "Smart Array P840", 0, 1},
+	{ 0x21CC103C, "Smart Array P542t", 0, 1},
+	{ 0x21CD103C, "Smart Array P240tr", 0, 1},
+	{ 0x21CE103C, "Smart HBA H240tr", 0, 1},
 
 
 #ifdef HAVE_SCSI_SG_H
@@ -620,7 +639,7 @@ static void find_bus_target(struct identify_controller *id, int bmic_drive_numbe
 	int big_map_support;
 	int drives_per_scsi_bus;
 
-	big_map_support = (id->controller_flags && (1<<7));
+	big_map_support = (id->controller_flags & (1<<7));
 	drives_per_scsi_bus = id->drives_per_scsi_bus;;
 
 	/* if no big drive map support, & zero devices per scsi bus */
@@ -666,7 +685,7 @@ static void format_phys_drive_location(char *location, int bus, int target,
 		copy_drive_field(model, device_data->drive_model, sizeof(device_data->drive_model));
 		copy_drive_field(serial_no, device_data->drive_serial_no, sizeof(device_data->drive_serial_no));
 		copy_drive_field(fw_rev, device_data->drive_fw_rev, sizeof(device_data->drive_fw_rev));
-		sprintf(tail, " connector %c%c box %d bay %d %40s %40s %8s",
+		sprintf(tail, " connector %c%c box %d bay %-2d %40s %40s %8s",
 			device_data->phys_connector[0],
 			device_data->phys_connector[1],
 			device_data->phys_box_on_bus,
@@ -984,7 +1003,7 @@ static void show_disk_map(char *title, char *filename, int fd,
 	struct identify_physical_device device_data;
 	char location[300];
 
-	big_map_support = (id->controller_flags && (1<<7));
+	big_map_support = (id->controller_flags & (1<<7));
 	drives_per_scsi_bus = id->drives_per_scsi_bus;;
 
 	/* if no big drive map support, & zero devices per scsi bus */
@@ -1081,12 +1100,14 @@ static void print_volume_status(char *file, int fd, int ctlrtype,
 			break;
 		case 1: sprintf(raid_level, "RAID 4");
 			break;
-		case 2: sprintf(raid_level, "RAID 1");
+		case 2: sprintf(raid_level, "RAID 1(1+0)");
 			break;
 		case 3: sprintf(raid_level, "RAID 5");
 			break;
 		case 5: sprintf(raid_level, "RAID 6");
 			break;
+		case 6: sprintf(raid_level, "RAID 1 ADM");
+			break;
 		default: /* mysteriously we don't know (shouldn't get here.) */
 			sprintf(raid_level, "(Unknown RAID level (tolerance_type = %d)",
 				tolerance_type);
@@ -1185,7 +1206,7 @@ static void print_bus_status(char *file, int ctlrtype,
 	/* Eg. 6400 with internal card cage gives back bogus info */
 	/* for me for the serial number. */
 	strncpy(enclosure_sn, bus_param->chassis_sn, 40);
-	enclosure_sn[41] = '\0';
+	enclosure_sn[40] = '\0';
 	for (i = 40 - 1; i > 0; i--)
 		if (enclosure_sn[i] == ' ') {
 			enclosure_sn[i] = '\0';
@@ -2271,6 +2292,8 @@ static void check_one_nonvolatile_cache_status(char *file, int ctlrtype,
 	if (cache_config->disable_flag)
 		return;
 
+	printf("%35s: %hu MiB\n", "Total cache memory", cache_config->total_memory_size);
+	printf("%35s: %d%% Read / %d%% Write\n", "Cache Ratio", cache_config->percent_read_cache, cache_config->percent_write_cache);
 	printf("%35s: %hu MiB\n", "Read cache memory", cache_config->mem_for_read_cache);
 	printf("%35s: %hu MiB\n", "Write cache memory", cache_config->mem_for_write_cache);
 	printf("%35s: %s\n", "Write cache enabled",
@@ -2440,11 +2463,13 @@ static void cciss_logical_drive_status(char *file, int fd,
 				volume_number);
 	}
 	scsi_device_node = file;
+#ifdef HAVE_SCSI_SG_H
 	if (is_scsi)
 		find_scsi_device_node(cciss_to_bmic.addr[volume_number].bmic_id_ctlr_data,
 			&scsi_device_node);
 	else
 		scsi_device_node = file;
+#endif
 	print_volume_status(scsi_device_node, fd, ctlrtype,
 		cciss_to_bmic.addr[volume_number].controller_lun,
 		volume_number, &ldstatus, id,
@@ -2454,7 +2479,7 @@ static void cciss_logical_drive_status(char *file, int fd,
 
 static inline int bmic_supports_big_maps(struct identify_controller *id)
 {
-	return id->controller_flags && (1<<7);
+	return id->controller_flags & (1<<7);
 }
 
 static int bmic_next_disk_bits(uint8_t *bits, int bitmapsize, int disk)
